@@ -23,7 +23,7 @@ import re, os
 
 
 # The current version of rdiff-backup
-version = "0.11.1"
+version = "0.11.2"
 
 # If this is set, use this value in seconds as the current time
 # instead of reading it from the clock.
@@ -37,6 +37,11 @@ blocksize = 32768
 # values may save on connection overhead and latency.
 conn_bufsize = 98304
 
+# This is used in rorpiter.CacheIndexable.  The number represents the
+# number of rpaths which may be stuck in buffers when moving over a
+# remote connection.
+pipeline_max_length = int(conn_bufsize / 150)*2
+
 # True if script is running as a server
 server = None
 
@@ -99,22 +104,15 @@ client_conn = None
 # list.
 changed_settings = []
 
-# The RPath of the rdiff-backup-data directory.
+# The RPath or QuotedRPath of the rdiff-backup-data directory.
 rbdir = None
 
-# This string is used when recognizing and creating time strings.
-# If the time_separator is ":", then W3 datetime strings like
-# 2001-12-07T04:22:01-07:00 are produced.  It can be set to "_" to
-# make filenames that don't contain colons, which aren't allowed
-# under MS windows NT.
-time_separator = ":"
-
 # quoting_enabled is true if we should quote certain characters in
 # filenames on the source side (see FilenameMapping for more
 # info).  chars_to_quote is a string whose characters should be
 # quoted, and quoting_char is the character to quote with.
 quoting_enabled = None
-chars_to_quote = ""
+chars_to_quote = "A-Z:"
 quoting_char = ';'
 
 # If true, emit output intended to be easily readable by a
@@ -173,6 +171,10 @@ security_level = "all"
 # deal with paths inside of restrict_path.
 restrict_path = None
 
+# If set, a file will be marked as changed if its inode changes.  See
+# the man page under --no-compare-inode for more information.
+compare_inode = 1
+
 
 def get(name):
 	"""Return the value of something in this module"""
