@@ -1,4 +1,3 @@
-
 # Copyright 2002 Ben Escoto
 #
 # This file is part of rdiff-backup.
@@ -22,10 +21,10 @@
 
 from __future__ import generators
 import getopt, sys, re, os
-from log import Log, LoggerError
+from log import Log, LoggerError, ErrorLog
 import Globals, Time, SetConnections, selection, robust, rpath, \
 	   manage, backup, connection, restore, FilenameMapping, \
-	   Security, Hardlink
+	   Security, Hardlink, regress, C
 
 
 action = None
@@ -45,14 +44,15 @@ def parse_cmdlineoptions(arglist):
 
 	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
 		 ["backup-mode", "calculate-average", "chars-to-quote=",
-		  "current-time=", "exclude=", "exclude-device-files",
-		  "exclude-filelist=", "exclude-filelist-stdin",
-		  "exclude-globbing-filelist=", "exclude-mirror=",
-		  "exclude-other-filesystems", "exclude-regexp=",
-		  "exclude-special-files", "force", "include=",
-		  "include-filelist=", "include-filelist-stdin",
+		  "check-destination-dir", "current-time=", "exclude=",
+		  "exclude-device-files", "exclude-filelist=",
+		  "exclude-filelist-stdin", "exclude-globbing-filelist=",
+		  "exclude-mirror=", "exclude-other-filesystems",
+		  "exclude-regexp=", "exclude-special-files", "force",
+		  "include=", "include-filelist=", "include-filelist-stdin",
 		  "include-globbing-filelist=", "include-regexp=",
-		  "list-changed-since=", "list-increments", "no-compression",
+		  "list-changed-since=", "list-increments",
+		  "no-compare-inode", "no-compression",
 		  "no-compression-regexp=", "no-hard-links", "null-separator",
 		  "parsable-output", "print-statistics", "quoting-char=",
 		  "remote-cmd=", "remote-schema=", "remove-older-than=",
@@ -67,6 +67,7 @@ def parse_cmdlineoptions(arglist):
 	for opt, arg in optlist:
 		if opt == "-b" or opt == "--backup-mode": action = "backup"
 		elif opt == "--calculate-average": action = "calculate-average"
+		elif opt == "--check-destination-dir": action = "check-destination-dir"
 		elif opt == "--chars-to-quote":
 			Globals.set('chars_to_quote', arg)
 			Globals.set('quoting_enabled', 1)
@@ -104,6 +105,7 @@ def parse_cmdlineoptions(arglist):
 			restore_timestr, action = arg, "list-changed-since"
 		elif opt == "-l" or opt == "--list-increments":
 			action = "list-increments"
+		elif opt == "--no-compare-inode": Globals.set("compare_inode", 0)
 		elif opt == "--no-compression": Globals.set("compression", None)
 		elif opt == "--no-compression-regexp":
 			Globals.set("no_compression_regexp_string", arg)
@@ -142,16 +144,21 @@ def parse_cmdlineoptions(arglist):
 			sys.exit(0)
 		elif opt == "-v" or opt == "--verbosity": Log.setverbosity(arg)
 		elif opt == "--windows-mode":
-			Globals.set('time_separator', "_")
 			Globals.set('chars_to_quote', "A-Z:")
 			Globals.set('quoting_enabled', 1)
 			Globals.set('preserve_hardlinks', 0)
-			select_opts.append(("--exclude-special-files", None))
-			assert 0, "Windows mode doesn't work in this version!"
-		elif opt == '--windows-time-format':
-			Globals.set('time_separator', "_")
 		else: Log.FatalError("Unknown option %s" % opt)
 
+def isincfilename(path):
+	"""Return true if path is of a (possibly quoted) increment file"""
+	rp = rpath.RPath(Globals.local_connection, path)
+	if Globals.quoting_enabled:
+		if not FilenameMapping.quoting_char:
+			FilenameMapping.set_init_quote_vals()
+		rp = FilenameMapping.get_quotedrpath(rp, separate_basename = 1)
+	result = rp.isincfile()
+	return result
+
 def set_action():
 	"""Check arguments and try to set action"""
 	global action
@@ -160,8 +167,7 @@ def set_action():
 		if l == 0: commandline_error("No arguments given")
 		elif l == 1: action = "restore"
 		elif l == 2:
-			if rpath.RPath(Globals.local_connection, args[0]).isincfile():
-				action = "restore"
+			if isincfilename(args[0]): action = "restore"
 			else: action = "backup"
 		else: commandline_error("Too many arguments given")
 
@@ -173,7 +179,8 @@ def set_action():
 		commandline_error("Two arguments are required (source, destination).")
 	if l == 2 and (action == "list-increments" or
 				   action == "remove-older-than" or
-				   action == "list-changed-since"):
+				   action == "list-changed-since" or
+				   action == "check-destination-dir"):
 		commandline_error("Only use one argument, "
 						  "the root of the backup directory")
 	if l > 2 and action != "calculate-average":
@@ -208,11 +215,13 @@ def take_action(rps):
 	elif action == "list-increments": ListIncrements(rps[0])
 	elif action == "remove-older-than": RemoveOlderThan(rps[0])
 	elif action == "calculate-average": CalculateAverage(rps)
+	elif action == "check-destination-dir": CheckDest(rps[0])
 	else: raise AssertionError("Unknown action " + action)
 
 def cleanup():
 	"""Do any last minute cleaning before exiting"""
 	Log("Cleaning up", 6)
+	if ErrorLog.isopen(): ErrorLog.close()
 	Log.close_logfile()
 	if not Globals.server: SetConnections.CloseConnections()
 
@@ -230,14 +239,19 @@ def Main(arglist):
 
 def Backup(rpin, rpout):
 	"""Backup, possibly incrementally, src_path to dest_path."""
+	if Globals.quoting_enabled:
+		rpout = FilenameMapping.get_quotedrpath(rpout)
 	SetConnections.BackupInitConnections(rpin.conn, rpout.conn)
 	backup_set_select(rpin)
 	backup_init_dirs(rpin, rpout)
 	if prevtime:
+		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
 		Time.setprevtime(prevtime)
 		backup.Mirror_and_increment(rpin, rpout, incdir)
-	else: backup.Mirror(rpin, rpout)
-	rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
+		rpout.conn.Main.backup_remove_curmirror_local()
+	else:
+		backup.Mirror(rpin, rpout)
+		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
 
 def backup_set_select(rpin):
 	"""Create Select objects on source connection"""
@@ -259,9 +273,10 @@ def backup_init_dirs(rpin, rpout):
 	elif not rpin.isdir():
 		Log.FatalError("Source %s is not a directory" % rpin.path)
 
-	datadir = rpout.append("rdiff-backup-data")
+	datadir = rpout.append_path("rdiff-backup-data")
 	SetConnections.UpdateGlobal('rbdir', datadir)
-	incdir = rpath.RPath(rpout.conn, os.path.join(datadir.path, "increments"))
+	checkdest_if_necessary(rpout)
+	incdir = datadir.append_path("increments")
 	prevtime = backup_get_mirrortime()
 
 	if rpout.lstat():
@@ -284,6 +299,7 @@ option.""" % rpout.path)
 	if not datadir.lstat(): datadir.mkdir()
 	if Log.verbosity > 0:
 		Log.open_logfile(datadir.append("backup.log"))
+	ErrorLog.open(Time.curtimestr, compress = Globals.compression)
 	backup_warn_if_infinite_regress(rpin, rpout)
 
 def backup_warn_if_infinite_regress(rpin, rpout):
@@ -300,39 +316,45 @@ def backup_warn_if_infinite_regress(rpin, rpout):
 source directory '%s'.  This could cause an infinite regress.  You
 may need to use the --exclude option.""" % (rpout.path, rpin.path), 2)
 
-def backup_get_mirrorrps():
-	"""Return list of current_mirror rps"""
-	datadir = Globals.rbdir
-	if not datadir.isdir(): return []
-	mirrorrps = [datadir.append(fn) for fn in datadir.listdir()
-				 if fn.startswith("current_mirror.")]
-	return filter(lambda rp: rp.isincfile(), mirrorrps)
-
 def backup_get_mirrortime():
 	"""Return time in seconds of previous mirror, or None if cannot"""
-	mirrorrps = backup_get_mirrorrps()
-	if not mirrorrps: return None
-	if len(mirrorrps) > 1:
-		Log(
-"""Warning: duplicate current_mirror files found.  Perhaps something
-went wrong during your last backup?  Using """ + mirrorrps[-1].path, 2)
-
-	return mirrorrps[-1].getinctime()
+	incbase = Globals.rbdir.append_path("current_mirror")
+	mirror_rps = restore.get_inclist(incbase)
+	assert len(mirror_rps) <= 1, \
+		   "Found %s current_mirror rps, expected <=1" % (len(mirror_rps),)
+	if mirror_rps: return mirror_rps[0].getinctime()
+	else: return None
 
 def backup_touch_curmirror_local(rpin, rpout):
 	"""Make a file like current_mirror.time.data to record time
 
-	Also updates rpout so mod times don't get messed up.  This should
-	be run on the destination connection.
+	When doing an incremental backup, this should happen before any
+	other writes, and the file should be removed after all writes.
+	That way we can tell whether the previous session aborted if there
+	are two current_mirror files.
+
+	When doing the initial full backup, the file can be created after
+	everything else is in place.
 
 	"""
-	datadir = Globals.rbdir
-	map(rpath.RPath.delete, backup_get_mirrorrps())
-	mirrorrp = datadir.append("current_mirror.%s.%s" % (Time.curtimestr,
-														"data"))
+	mirrorrp = Globals.rbdir.append("current_mirror.%s.%s" % (Time.curtimestr,
+															  "data"))
 	Log("Touching mirror marker %s" % mirrorrp.path, 6)
 	mirrorrp.touch()
-	rpath.copy_attribs(rpin, rpout)
+	mirrorrp.fsync_with_dir()
+
+def backup_remove_curmirror_local():
+	"""Remove the older of the current_mirror files.  Use at end of session"""
+	assert Globals.rbdir.conn is Globals.local_connection
+	curmir_incs = restore.get_inclist(Globals.rbdir.append("current_mirror"))
+	assert len(curmir_incs) == 2
+	if curmir_incs[0].getinctime() < curmir_incs[1].getinctime():
+		older_inc = curmir_incs[0]
+	else: older_inc = curmir_incs[1]
+
+	C.sync() # Make sure everything is written before curmirror is removed
+	older_inc.sync_delete()
+
 
 def Restore(src_rp, dest_rp = None):
 	"""Main restoring function
@@ -351,7 +373,7 @@ def RestoreAsOf(rpin, target):
 	target - RPath of place to put restored file
 
 	"""
-	restore_check_paths(rpin, target, 1)
+	rpin, rpout = restore_check_paths(rpin, target, 1)
 	try: time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
 	restore_common(rpin, target, time)
@@ -361,6 +383,7 @@ def restore_common(rpin, target, time):
 	if target.conn.os.getuid() == 0:
 		SetConnections.UpdateGlobal('change_ownership', 1)
 	mirror_root, index = restore_get_root(rpin)
+	restore_check_backup_dir(mirror_root)
 	mirror = mirror_root.new_index(index)
 	inc_rpath = datadir.append_path('increments', index)
 	restore_init_select(mirror_root, target)
@@ -384,7 +407,9 @@ def restore_check_paths(rpin, rpout, restoreasof = None):
 	if not restoreasof:
 		if not rpin.lstat():
 			Log.FatalError("Source file %s does not exist" % rpin.path)
-		elif not rpin.isincfile():
+		if Globals.quoting_enabled:
+			rpin = FilenameMapping.get_quotedrpath(rpin, 1)
+		if not rpin.isincfile():
 			Log.FatalError("""File %s does not look like an increment file.
 
 Try restoring from an increment file (the filenames look like
@@ -397,6 +422,17 @@ Try restoring from an increment file (the filenames look like
 					   "specify --force to overwrite." % rpout.path)
 	return rpin, rpout
 
+def restore_check_backup_dir(rpin):
+	"""Make sure backup dir root rpin is in consistent state"""
+	result = checkdest_need_check(rpin)
+	if result is None:
+		Log.FatalError("%s does not appear to be an rdiff-backup directory."
+					   % (rpin.path,))
+	elif result == 1: Log.FatalError(
+		"Previous backup to %s seems to have failed."
+		"Rerun rdiff-backup with --check-destination-dir option to revert"
+		"directory to state before unsuccessful session." % (rpin.path,))
+
 def restore_init_select(rpin, rpout):
 	"""Initialize Select
 
@@ -438,7 +474,8 @@ def restore_get_root(rpin):
 		i = i-1
 	else: Log.FatalError("Unable to find rdiff-backup-data directory")
 
-	rootrp = parent_dir
+	if not Globals.quoting_enabled: rootrp = parent_dir
+	else: rootrp = FilenameMapping.get_quotedrpath(parent_dir)
 	Log("Using mirror root directory %s" % rootrp.path, 6)
 
 	datadir = rootrp.append_path("rdiff-backup-data")
@@ -457,6 +494,7 @@ def restore_get_root(rpin):
 def ListIncrements(rp):
 	"""Print out a summary of the increments and their times"""
 	mirror_root, index = restore_get_root(rp)
+	restore_check_backup_dir(mirror_root)
 	mirror_rp = mirror_root.new_index(index)
 	inc_rpath = Globals.rbdir.append_path('increments', index)
 	incs = restore.get_inclist(inc_rpath)
@@ -476,21 +514,18 @@ def CalculateAverage(rps):
 
 def RemoveOlderThan(rootrp):
 	"""Remove all increment files older than a certain time"""
-	datadir = rootrp.append("rdiff-backup-data")
-	if not datadir.lstat() or not datadir.isdir():
-		Log.FatalError("Unable to open rdiff-backup-data dir %s" %
-					   (datadir.path,))
-
+	rom_check_dir(rootrp)
 	try: time = Time.genstrtotime(remove_older_than_string)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
 	timep = Time.timetopretty(time)
 	Log("Deleting increment(s) before %s" % timep, 4)
 
 	times_in_secs = [inc.getinctime() for inc in 
-					 restore.get_inclist(datadir.append("increments"))]
+		  restore.get_inclist(Globals.rbdir.append_path("increments"))]
 	times_in_secs = filter(lambda t: t < time, times_in_secs)
 	if not times_in_secs:
-		Log.FatalError("No increments older than %s found" % timep)
+		Log.FatalError("No increments older than %s found, exiting."
+					   % (timep,), 1)
 
 	times_in_secs.sort()
 	inc_pretty_time = "\n".join(map(Time.timetopretty, times_in_secs))
@@ -502,7 +537,16 @@ def RemoveOlderThan(rootrp):
 	if len(times_in_secs) == 1:
 		Log("Deleting increment at time:\n" + inc_pretty_time, 3)
 	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
-	manage.delete_earlier_than(datadir, time)
+	manage.delete_earlier_than(Globals.rbdir, time)
+
+def rom_check_dir(rootrp):
+	"""Check destination dir before RemoveOlderThan"""
+	SetConnections.UpdateGlobal('rbdir',
+								rootrp.append_path("rdiff-backup-data"))
+	if not Globals.rbdir.isdir():
+		Log.FatalError("Unable to open rdiff-backup-data dir %s" %
+					   (datadir.path,))
+	checkdest_if_necessary(rootrp)
 
 
 def ListChangedSince(rp):
@@ -510,7 +554,52 @@ def ListChangedSince(rp):
 	try: rest_time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
 	mirror_root, index = restore_get_root(rp)
+	restore_check_backup_dir(mirror_root)
 	mirror_rp = mirror_root.new_index(index)
 	inc_rp = mirror_rp.append_path("increments", index)
 	restore.ListChangedSince(mirror_rp, inc_rp, rest_time)
 
+
+def CheckDest(dest_rp):
+	"""Check the destination directory, """
+	if Globals.rbdir is None:
+		SetConnections.UpdateGlobal('rbdir',
+									dest_rp.append_path("rdiff-backup-data"))
+	need_check = checkdest_need_check(dest_rp)
+	if need_check is None:
+		Log.FatalError("No destination dir found at %s" % (dest_rp.path,))
+	elif need_check == 0:
+		Log.FatalError("Destination dir %s does not need checking" %
+					   (dest_rp.path,))
+	dest_rp.conn.regress.Regress(dest_rp)
+
+def checkdest_need_check(dest_rp):
+	"""Return None if no dest dir found, 1 if dest dir needs check, 0 o/w"""
+	if not dest_rp.isdir() or not Globals.rbdir.isdir(): return None
+	curmirroot = Globals.rbdir.append("current_mirror")
+	curmir_incs = restore.get_inclist(curmirroot)
+	if not curmir_incs:
+		Log.FatalError(
+"""Bad rdiff-backup-data dir on destination side
+
+The rdiff-backup data directory
+%s
+exists, but we cannot find a valid current_mirror marker.  You can
+avoid this message by removing this directory; however any data in it
+will be lost.
+""" % (Globals.rbdir.path,))
+	elif len(curmir_incs) == 1: return 0
+	else:
+		assert len(curmir_incs) == 2, "Found too many current_mirror incs!"
+		return 1
+
+def checkdest_if_necessary(dest_rp):
+	"""Check the destination dir if necessary.
+
+	This can/should be run before an incremental backup.
+
+	"""
+	need_check = checkdest_need_check(dest_rp)
+	if need_check == 1:
+		Log("Previous backup seems to have failed, checking now.", 2)
+		dest_rp.conn.regress.Regress(dest_rp)
