@@ -21,8 +21,8 @@
 
 from __future__ import generators
 import tempfile, os
-import Globals, Time, Rdiff, Hardlink, FilenameMapping, rorpiter, \
-	   selection, rpath, log, backup, static, robust, metadata
+import Globals, Time, Rdiff, Hardlink, rorpiter, selection, rpath, \
+	   log, static, robust, metadata, statistics, TempFile
 
 
 # This should be set to selection.Select objects over the source and
@@ -52,20 +52,13 @@ def Restore(mirror_rp, inc_rpath, target, restore_to_time):
 def get_inclist(inc_rpath):
 	"""Returns increments with given base"""
 	dirname, basename = inc_rpath.dirsplit()
-	parent_dir = rpath.RPath(inc_rpath.conn, dirname, ())
+	parent_dir = inc_rpath.__class__(inc_rpath.conn, dirname, ())
 	if not parent_dir.isdir(): return [] # inc directory not created yet
 	index = inc_rpath.index
 
-	if index:
-		get_inc_ext = lambda filename: \
-					  rpath.RPath(inc_rpath.conn, inc_rpath.base,
-								  inc_rpath.index[:-1] + (filename,))
-	else: get_inc_ext = lambda filename: \
-			 rpath.RPath(inc_rpath.conn, os.path.join(dirname, filename))
-
 	inc_list = []
 	for filename in parent_dir.listdir():
-		inc = get_inc_ext(filename)
+		inc = parent_dir.append(filename)
 		if inc.isincfile() and inc.getincbase_str() == basename:
 			inc_list.append(inc)
 	return inc_list
@@ -227,11 +220,18 @@ class TargetStruct:
 	def patch(cls, target, diff_iter):
 		"""Patch target with the diffs from the mirror side
 
-		This function was already written for use when backing up, so
-		just use that.
+		This function and the associated ITRB is similar to the
+		patching code in backup.py, but they have different error
+		correction requirements, so it seemed easier to just repeat it
+		all in this module.
 
 		"""
-		backup.DestinationStruct.patch(target, diff_iter)
+		ITR = rorpiter.IterTreeReducer(PatchITRB, [target])
+		for diff in rorpiter.FillInIter(diff_iter, target):
+			log.Log("Processing changed file " + diff.get_indexpath(), 5)
+			ITR(diff.index, diff)
+		ITR.Finish()
+		target.setdata()
 
 static.MakeClass(TargetStruct)
 
@@ -286,6 +286,7 @@ class CachedRF:
 		assert new_rfs, "No RFs added for index %s" % index
 		self.rf_list[0:0] = new_rfs
 
+
 class RestoreFile:
 	"""Hold data about a single mirror file and its related increments
 
@@ -368,7 +369,7 @@ class RestoreFile:
 
 	def get_restore_fp(self):
 		"""Return file object of restored data"""
-		assert self.relevant_incs[-1].isreg(), "Not a regular file"
+		assert self.relevant_incs[-1].isreg()
 		current_fp = self.get_first_fp()
 		for inc_diff in self.relevant_incs[1:]:
 			log.Log("Applying patch %s" % (inc_diff.get_indexpath(),), 7)
@@ -397,33 +398,29 @@ class RestoreFile:
 	def yield_sub_rfs(self):
 		"""Return RestoreFiles under current RestoreFile (which is dir)"""
 		assert self.mirror_rp.isdir() or self.inc_rp.isdir()
-		mirror_iter = self.yield_mirrorrps(self.mirror_rp)
-		inc_pair_iter = self.yield_inc_complexes(self.inc_rp)
+		if self.mirror_rp.isdir():
+			mirror_iter = self.yield_mirrorrps(self.mirror_rp)
+		else: mirror_iter = iter([])
+		if self.inc_rp.isdir():
+			inc_pair_iter = self.yield_inc_complexes(self.inc_rp)
+		else: inc_pair_iter = iter([])
 		collated = rorpiter.Collate2Iters(mirror_iter, inc_pair_iter)
 
 		for mirror_rp, inc_pair in collated:
 			if not inc_pair:
 				inc_rp = self.inc_rp.new_index(mirror_rp.index)
-				if Globals.quoting_enabled: inc_rp.quote_path()
 				inc_list = []
 			else: inc_rp, inc_list = inc_pair
 			if not mirror_rp:
 				mirror_rp = self.mirror_rp.new_index(inc_rp.index)
-				if Globals.quoting_enabled: mirror_rp.quote_path()
-			yield RestoreFile(mirror_rp, inc_rp, inc_list)
+			yield self.__class__(mirror_rp, inc_rp, inc_list)
 
 	def yield_mirrorrps(self, mirrorrp):
 		"""Yield mirrorrps underneath given mirrorrp"""
-		if mirrorrp and mirrorrp.isdir():
-			if Globals.quoting_enabled:
-				for rp in selection.get_quoted_dir_children(mirrorrp):
-					if rp.index != ('rdiff-backup-data',): yield rp
-			else:
-				dirlist = mirrorrp.listdir()
-				dirlist.sort()
-				for filename in dirlist:
-					rp = mirrorrp.append(filename)
-					if rp.index != ('rdiff-backup-data',): yield rp
+		assert mirrorrp.isdir()
+		for filename in robust.listrp(mirrorrp):
+			rp = mirrorrp.append(filename)
+			if rp.index != ('rdiff-backup-data',): yield rp
 
 	def yield_inc_complexes(self, inc_rpath):
 		"""Yield (sub_inc_rpath, inc_list) IndexedTuples from given inc_rpath
@@ -433,23 +430,19 @@ class RestoreFile:
 
 		"""
 		if not inc_rpath.isdir(): return
-		inc_dict = {} # dictionary of basenames:inc_lists
+		inc_dict = {} # dictionary of basenames:IndexedTuples(index, inc_list)
 		dirlist = robust.listrp(inc_rpath)
-		if Globals.quoting_enabled:
-			dirlist = [FilenameMapping.unquote(fn) for fn in dirlist]
 
 		def affirm_dict_indexed(basename):
 			"""Make sure the rid dictionary has given basename as key"""
 			if not inc_dict.has_key(basename):
 				sub_inc_rp = inc_rpath.append(basename)
-				if Globals.quoting_enabled: sub_inc_rp.quote_path()
 				inc_dict[basename] = rorpiter.IndexedTuple(sub_inc_rp.index,
 														   (sub_inc_rp, []))
 
 		def add_to_dict(filename):
 			"""Add filename to the inc tuple dictionary"""
 			rp = inc_rpath.append(filename)
-			if Globals.quoting_enabled: rp.quote_path()
 			if rp.isincfile() and rp.getinctype() != 'data':
 				basename = rp.getincbase_str()
 				affirm_dict_indexed(basename)
@@ -460,3 +453,90 @@ class RestoreFile:
 		keys = inc_dict.keys()
 		keys.sort()
 		for key in keys: yield inc_dict[key]
+
+
+class PatchITRB(rorpiter.ITRBranch):
+	"""Patch an rpath with the given diff iters (use with IterTreeReducer)
+
+	The main complication here involves directories.  We have to
+	finish processing the directory after what's in the directory, as
+	the directory may have inappropriate permissions to alter the
+	contents or the dir's mtime could change as we change the
+	contents.
+
+	This code was originally taken from backup.py.  However, because
+	of different error correction requirements, it is repeated here.
+
+	"""
+	def __init__(self, basis_root_rp):
+		"""Set basis_root_rp, the base of the tree to be incremented"""
+		self.basis_root_rp = basis_root_rp
+		assert basis_root_rp.conn is Globals.local_connection
+		self.dir_replacement, self.dir_update = None, None
+		self.cached_rp = None
+
+	def get_rp_from_root(self, index):
+		"""Return RPath by adding index to self.basis_root_rp"""
+		if not self.cached_rp or self.cached_rp.index != index:
+			self.cached_rp = self.basis_root_rp.new_index(index)
+		return self.cached_rp
+
+	def can_fast_process(self, index, diff_rorp):
+		"""True if diff_rorp and mirror are not directories"""
+		rp = self.get_rp_from_root(index)
+		return not diff_rorp.isdir() and not rp.isdir()
+
+	def fast_process(self, index, diff_rorp):
+		"""Patch base_rp with diff_rorp (case where neither is directory)"""
+		rp = self.get_rp_from_root(index)
+		tf = TempFile.new(rp)
+		self.patch_to_temp(rp, diff_rorp, tf)
+		rpath.rename(tf, rp)
+
+	def patch_to_temp(self, basis_rp, diff_rorp, new):
+		"""Patch basis_rp, writing output in new, which doesn't exist yet"""
+		if diff_rorp.isflaglinked():
+			Hardlink.link_rp(diff_rorp, new, self.basis_root_rp)
+		elif diff_rorp.get_attached_filetype() == 'snapshot':
+			rpath.copy(diff_rorp, new)
+		else:
+			assert diff_rorp.get_attached_filetype() == 'diff'
+			Rdiff.patch_local(basis_rp, diff_rorp, new)
+		if new.lstat(): rpath.copy_attribs(diff_rorp, new)
+
+	def start_process(self, index, diff_rorp):
+		"""Start processing directory - record information for later"""
+		base_rp = self.base_rp = self.get_rp_from_root(index)
+		assert diff_rorp.isdir() or base_rp.isdir() or not base_rp.index
+		if diff_rorp.isdir(): self.prepare_dir(diff_rorp, base_rp)
+		else: self.set_dir_replacement(diff_rorp, base_rp)
+
+	def set_dir_replacement(self, diff_rorp, base_rp):
+		"""Set self.dir_replacement, which holds data until done with dir
+
+		This is used when base_rp is a dir, and diff_rorp is not.
+
+		"""
+		assert diff_rorp.get_attached_filetype() == 'snapshot'
+		self.dir_replacement = TempFile.new(base_rp)
+		rpath.copy_with_attribs(diff_rorp, self.dir_replacement)
+		if base_rp.isdir(): base_rp.chmod(0700)
+
+	def prepare_dir(self, diff_rorp, base_rp):
+		"""Prepare base_rp to turn into a directory"""
+		self.dir_update = diff_rorp.getRORPath() # make copy in case changes
+		if not base_rp.isdir():
+			if base_rp.lstat(): base_rp.delete()
+			base_rp.mkdir()
+		base_rp.chmod(0700)
+
+	def end_process(self):
+		"""Finish processing directory"""
+		if self.dir_update:
+			assert self.base_rp.isdir()
+			rpath.copy_attribs(self.dir_update, self.base_rp)
+		else:
+			assert self.dir_replacement
+			self.base_rp.rmdir()
+			if self.dir_replacement.lstat():
+				rpath.rename(self.dir_replacement, self.base_rp)
