@@ -36,7 +36,7 @@ are dealing with are local or remote.
 """
 
 import os, stat, re, sys, shutil, gzip, socket, time
-import Globals, FilenameMapping, Time, static, log
+import Globals, Time, static, log
 
 
 class SkipFileException(Exception):
@@ -90,7 +90,7 @@ def copy(rpin, rpout, compress = 0):
 
 	if rpout.lstat():
 		if rpin.isreg() or not cmp(rpin, rpout):
-			rpout.delete()   # easier to write that compare
+			rpout.delete()   # easier to write than compare
 		else: return
 
 	if rpin.isreg(): copy_reg_file(rpin, rpout, compress)
@@ -177,7 +177,7 @@ def cmp_attribs(rp1, rp2):
 	elif rp1.ischardev() and rp2.ischardev(): result = 1
 	else: result = (rp1.getmtime() == rp2.getmtime())
 	log.Log("Compare attribs of %s and %s: %s" %
-			(rp1.path, rp2.path, result), 7)
+			(rp1.get_indexpath(), rp2.get_indexpath(), result), 7)
 	return result
 
 def copy_with_attribs(rpin, rpout, compress = 0):
@@ -185,22 +185,6 @@ def copy_with_attribs(rpin, rpout, compress = 0):
 	copy(rpin, rpout, compress)
 	if rpin.lstat(): copy_attribs(rpin, rpout)
 
-def quick_cmp_with_attribs(rp1, rp2):
-	"""Quicker version of cmp_with_attribs
-
-	Instead of reading all of each file, assume that regular files
-	are the same if the attributes compare.
-
-	"""
-	if not cmp_attribs(rp1, rp2): return None
-	if rp1.isreg() and rp2.isreg() and (rp1.getlen() == rp2.getlen()):
-		return 1
-	return cmp(rp1, rp2)
-
-def cmp_with_attribs(rp1, rp2):
-	"""Combine cmp and cmp_attribs"""
-	return cmp_attribs(rp1, rp2) and cmp(rp1, rp2)
-
 def rename(rp_source, rp_dest):
 	"""Rename rp_source to rp_dest"""
 	assert rp_source.conn is rp_dest.conn
@@ -208,7 +192,12 @@ def rename(rp_source, rp_dest):
 			(rp_source.path, rp_dest.path), 7)
 	if not rp_source.lstat(): rp_dest.delete()
 	else:
-		rp_source.conn.os.rename(rp_source.path, rp_dest.path)
+		if rp_dest.lstat() and rp_source.getinode() == rp_dest.getinode():
+			log.Log("Warning: Attempt to rename over same inode: %s to %s"
+					% (rp_source.path, rp_dest.path), 2)
+			# You can't rename one hard linked file over another
+			rp_source.delete()
+		else: rp_source.conn.os.rename(rp_source.path, rp_dest.path)
 		rp_dest.data = rp_source.data
 		rp_source.data = {'type': None}
 
@@ -262,24 +251,60 @@ class RORPath:
 		else: self.data = {'type':None} # signify empty file
 		self.file = None
 
+	def zero(self):
+		"""Set inside of self to type None"""
+		self.data = {'type': None}
+		self.file = None
+
+	def __nonzero__(self): return 1
+
 	def __eq__(self, other):
 		"""True iff the two rorpaths are equivalent"""
 		if self.index != other.index: return None
 
 		for key in self.data.keys(): # compare dicts key by key
-			if ((key == 'uid' or key == 'gid') and
-				(not Globals.change_ownership or self.issym())):
-				# Don't compare gid/uid for symlinks or if not change_ownership
+			if (key == 'uid' or key == 'gid') and self.issym():
+				# Don't compare gid/uid for symlinks
 				pass
 			elif key == 'atime' and not Globals.preserve_atime: pass
-			elif key == 'devloc' or key == 'inode' or key == 'nlink': pass
-			elif key == 'size' and not self.isreg():
-				pass # size only matters for regular files
+			elif key == 'devloc' or key == 'nlink': pass
+			elif key == 'size' and not self.isreg(): pass
+			elif (key == 'inode' and
+				  (not self.isreg() or self.getnumlinks() == 1 or
+				   not Globals.compare_inode)): pass
 			elif (not other.data.has_key(key) or
 				  self.data[key] != other.data[key]): return None
 		return 1
 
-	def equal_verbose(self, other, check_index = 1):
+	def equal_loose(self, other):
+		"""True iff the two rorpaths are kinda equivalent
+
+		Sometimes because of missing permissions, a file cannot be
+		replicated exactly on the remote side.  This function tells
+		you whether the two files are close enough.  self must be the
+		file with more information.
+
+		"""
+		for key in self.data.keys(): # compare dicts key by key
+			if ((key == 'uid' or key == 'gid') and
+				(self.issym() or not Globals.change_ownership)):
+				# Don't compare gid/uid for symlinks, and only root
+				# can change ownership
+				pass
+			elif (key == 'type' and self.isspecial() and
+				  other.isreg() and other.getsize() == 0):
+				# Special files may be replaced with 0 len regular files
+				pass
+			elif key == 'atime' and not Globals.preserve_atime: pass
+			elif key == 'devloc' or key == 'nlink': pass
+			elif key == 'size' and not self.isreg(): pass
+			elif key == 'inode': pass
+			elif (not other.data.has_key(key) or
+				  self.data[key] != other.data[key]): return 0
+		return 1
+
+	def equal_verbose(self, other, check_index = 1,
+					  compare_inodes = 0, compare_ownership = 0):
 		"""Like __eq__, but log more information.  Useful when testing"""
 		if check_index and self.index != other.index:
 			log.Log("Index %s != index %s" % (self.index, other.index), 2)
@@ -287,12 +312,14 @@ class RORPath:
 
 		for key in self.data.keys(): # compare dicts key by key
 			if ((key == 'uid' or key == 'gid') and
-				(not Globals.change_ownership or self.issym())):
-				# Don't compare gid/uid for symlinks or if not change_ownership
+				(self.issym() or not compare_ownership)):
+				# Don't compare gid/uid for symlinks, or if told not to
 				pass
 			elif key == 'atime' and not Globals.preserve_atime: pass
-			elif key == 'devloc' or key == 'inode' or key == 'nlink': pass
+			elif key == 'devloc' or key == 'nlink': pass
 			elif key == 'size' and not self.isreg(): pass
+			elif key == 'inode' and (not self.isreg() or not compare_inodes):
+				pass
 			elif (not other.data.has_key(key) or
 				  self.data[key] != other.data[key]):
 				if not other.data.has_key(key):
@@ -308,6 +335,10 @@ class RORPath:
 		"""Pretty print file statistics"""
 		return "Index: %s\nData: %s" % (self.index, self.data)
 
+	def summary_string(self):
+		"""Return summary string"""
+		return "%s %s" % (self.get_indexpath(), self.lstat())
+
 	def __getstate__(self):
 		"""Return picklable state
 
@@ -369,6 +400,12 @@ class RORPath:
 		"""True if path is a socket"""
 		return self.data['type'] == 'sock'
 
+	def isspecial(self):
+		"""True if the file is a sock, symlink, device, or fifo"""
+		type = self.data['type']
+		return (type == 'dev' or type == 'sock' or
+				type == 'fifo' or type == 'sym')
+
 	def getperms(self):
 		"""Return permission block of file"""
 		return self.data['perms']
@@ -592,12 +629,6 @@ class RPath(RORPath):
 		s = self.conn.reval("lambda path: os.lstat(path).st_rdev", self.path)
 		return (s >> 8, s & 0xff)
 
-	def quote_path(self):
-		"""Set path from quoted version of index"""
-		quoted_list = [FilenameMapping.quote(path) for path in self.index]
-		self.path = "/".join([self.base] + quoted_list)
-		self.setdata()
-
 	def chmod(self, permissions):
 		"""Wrapper around os.chmod"""
 		self.conn.os.chmod(self.path, permissions)
@@ -664,7 +695,7 @@ class RPath(RORPath):
 		log.Log("Touching " + self.path, 7)
 		self.conn.open(self.path, "w").close()
 		self.setdata()
-		assert self.isreg()
+		assert self.isreg(), self.path
 
 	def hasfullperms(self):
 		"""Return true if current process has full permissions on the file"""
@@ -696,11 +727,7 @@ class RPath(RORPath):
 	def delete(self):
 		"""Delete file at self.path.  Recursively deletes directories."""
 		log.Log("Deleting %s" % self.path, 7)
-		self.setdata()
-		if not self.lstat():
-			log.Log("Warning: %s does not exist---deleted in meantime?"
-				% (self.path,), 2)
-		elif self.isdir():
+		if self.isdir():
 			try: self.rmdir()
 			except os.error: shutil.rmtree(self.path)
 		else: self.conn.os.unlink(self.path)
@@ -742,6 +769,14 @@ class RPath(RORPath):
 		comps = normed.path.split("/")
 		return "/".join(comps[:-1]), comps[-1]
 
+	def get_parent_rp(self):
+		"""Return new RPath of directory self is in"""
+		if self.index:
+			return self.__class__(self.conn, self.base, self.index[:-1])
+		dirname = self.dirsplit()[0]
+		if dirname: return self.__class__(self.conn, dirname)
+		else: return self.__class__(self.conn, "/")
+
 	def newpath(self, newpath, index = ()):
 		"""Return new RPath with the same connection but different path"""
 		return self.__class__(self.conn, newpath, index)
@@ -843,7 +878,9 @@ class RPath(RORPath):
 
 	def getincbase_str(self):
 		"""Return the base filename string of an increment file"""
-		return self.getincbase().dirsplit()[1]
+		rp = self.getincbase()
+		if rp.index: return rp.index[-1]
+		else: return rp.dirsplit()[1]
 
 	def makedev(self, type, major, minor):
 		"""Make a special file with specified type, and major/minor nums"""
@@ -855,6 +892,45 @@ class RPath(RORPath):
 		else: raise RPathException
 		self.setdata()
 
+	def fsync(self, fp = None):
+		"""fsync the current file or directory
+
+		If fp is none, get the file description by opening the file.
+		This can be useful for directories.
+
+		"""
+		if not fp:
+			fp = self.open("rb")
+			os.fsync(fp.fileno())
+			assert not fp.close()
+		else: os.fsync(fp.fileno())
+
+	def fsync_with_dir(self, fp = None):
+		"""fsync self and directory self is under"""
+		self.fsync(fp)
+		self.get_parent_rp().fsync()
+
+	def sync_delete(self):
+		"""Delete self with sync to guarantee completion
+
+		On some filesystems (like linux's ext2), we must sync both the
+		file and the directory to make sure.
+
+		"""
+		if self.lstat() and not self.issym():
+			fp = self.open("rb")
+			self.delete()
+			os.fsync(fp.fileno())
+		assert not fp.close()
+		self.get_parent_rp().fsync()
+
+	def get_data(self):
+		"""Open file as a regular file, read data, close, return data"""
+		fp = self.open("rb")
+		s = fp.read()
+		assert not fp.close()
+		return s
+
 
 class RPathFileHook:
 	"""Look like a file, but add closing hook"""
@@ -871,4 +947,3 @@ class RPathFileHook:
 		self.closing_thunk()
 		return result
 
-	
